/* 
set constants
constants are used for easy reference when used in the code 

pins 5-6 and 10-11 are pulse width modulation (PWM) pins
pins A0-A2 are analogue input pins 
*/
const int greenLEDPin = 11;
const int redLEDPin = 10;  
const int blueLEDPin = 6; 
const int yellowLEDPin = 5;


const int redSensorPin = A0; 
const int greenSensorPin = A1; 
const int blueSensorPin = A2;

/*
There is where we declare variables. Variables will have their value change as the code runs.
The purpose of declaring variables is two fold:
We must create a space in the memory of the Arduino to store the information.
We must tell Arduino what type of information we are storing. 
In this document all are variables are integers but by using the keyword float instead of int we can make the variables floating type (that is decimal numbers).
*/
int redValue = 0;
int greenValue = 0;
int blueValue = 0;

int redSensorValue = 0;
int greenSensorValue = 0;
int blueSensorValue = 0;

int redGreenChannel = 0;
int blueYellowChannel = 0;

/*
The setup runs once, when the Arduino is first powered on. 
This is where you configure the digital pins to be either inputs or outputs using a function named pinMode. 

In addition we also open what is called a serial port. All that you need to know about serial ports is that the establish a  connection between the computer and the Arduino. Using the serial port we can get text information from the Arduino micro controller. To view the information given by the serial port click the magnifying glass icon in the Arduino IDE.
*/
void setup() {    
  Serial.begin (9600); // open a serial port and set the speed of port to have data transfer to the computer at 9600 bits per second
  
  /* 
  This first argument of pinMode is the number of the pin being used the second is wether the pin is an output or an input.  
  Each of the pwm pins is set to output. 
  Not that for this program in is not necessary to declare which pins are inputs.
  */
  pinMode (greenLEDPin, OUTPUT);
  pinMode (redLEDPin, OUTPUT);
  pinMode (blueLEDPin, OUTPUT);
} // end setup

/*
This part of the code runs continuously until power stops flowing into the Arduino. 
The purpose of this code is to
1) Get input from three different sensors.
2) Give information to the serial port, so that you can see the value of the variables that are stored on the Arduino microcontroller. 
3) Light up the LEDs based on the oppenent process theory.
*/
void loop() {  
  /*
  The purpose of this block of code is to 
  1) Get input from three different sensors.
  */
  redSensorValue = analogRead (redSensorPin); // Read red photoresistor  
  delay (5); //delay is used as reading takes some time

  greenSensorValue = analogRead (greenSensorPin); // Read green photoresistor 
  delay (5); //delay is used as reading takes some time  
  
  blueSensorValue = analogRead (blueSensorPin); // Read blue photoresistor 

  // Print the following to the serial port. The \t is a tab.
  
  Serial.print ("Raw Sensor Values \t Red: "); 
  Serial.print (redSensorValue); 
  Serial.print ("\t Green: "); 
  Serial.print (greenSensorValue);   
  Serial.print ("\t Blue: "); 
  Serial.println (blueSensorValue);   
  
  /*
  The purpose of this block of code is to 
  2) Give information to the serial port, so that you can see the value of the variables that are stored on the Arduino microcontroller.
  */
  // analogRead gives values between 0-1023, however analogWrite only accepts values between 0-255. 
  // The values must be divided by 4 to solve this problem
  redValue = redSensorValue/4;
  greenValue = greenSensorValue/4;
  blueValue = blueSensorValue/4;
  
  // Prints the following to serial port
  Serial.print ("Altered Sensor Values \t Red: "); 
  Serial.print (redValue); 
  Serial.print ("\t Green: "); 
  Serial.print (greenValue);   
  Serial.print ("\t blue: "); 
  Serial.println (blueValue);
  
  // applies opponent process theory to generate the two different colour channels
  redGreenChannel = redValue-greenValue;
  blueYellowChannel = redValue+greenValue-2*blueValue;
  
  // Prints the following to serial port
  Serial.print ("channel values \t Red-Green: "); 
  Serial.print (redGreenChannel); 
  Serial.print ("\t Blue-Yellow: "); 
  Serial.println (blueYellowChannel);   
  /* 
  The purpose of this block of code is to 
  3) Light up the LEDs based on the oppenent process theory.
  That is the colours of light will light up in the respone to the difference of light no the absloute amounts of light.
  Note that some of the voltages are muliplied by 1
  */
  
  if (redGreenChannel > 0){ // If red is the dominant colour 
    analogWrite (redLEDPin, redGreenChannel);     // red output 
  }
  else if (redGreenChannel < 0){ // If green is the dominant colour
  analogWrite (greenLEDPin, -1*redGreenChannel); // green output 
  }
  else{ // the inensity of green and red is relatively the same neither pin will light up
    analogWrite (greenLEDPin,0);
    analogWrite (redLEDPin,0);
  }
  
  if (blueYellowChannel > 0){ // if yellow is the dominant colour
    analogWrite (yellowLEDPin, blueYellowChannel); // yellow output 
  }
  else if ( blueYellowChannel < 0){ //if blue is dominant colour
  analogWrite (blueLEDPin, -1*blueYellowChannel); // blue output 
  }
  else{ // the inensity of blue and yellow is relatively the same neither pin will light up
    analogWrite (blueLEDPin,0);
    analogWrite (yellowLEDPin,0);
  }  
} // go back to the beginning of the loop
